@@ -351,7 +351,7 @@ from ae.core import DEBUG_LEVELS
 from ae.console import ConsoleApp                                                       # type: ignore
 
 
-__version__ = '0.1.51'
+__version__ = '0.1.52'
 
 
 AppStateType = Dict[str, Any]                   #: app state config variable type
@@ -392,22 +392,32 @@ FOUND_SOUNDS = FilesRegister()                  #: register of sound files found
 check_all()     # let ae_updater module check/install any outstanding updates or new app versions
 
 
-def id_of_flow(action: str, obj: str = "", key: str = "") -> str:
-    """ create flow id string.
+def ensure_tap_kwargs_refs(init_kwargs: Dict[str, Any], tap_widget: Any):
+    """ ensure that the passed widget.__init__ kwargs dict contains a reference to itself within kwargs['tap_kwargs'].
+
+    :param init_kwargs:         kwargs of the widgets __init__ method.
+    :param tap_widget:          reference to the tap widget.
+
+    This alternative version is only 10 % faster but much less clean than the current implementation::
+
+        if 'tap_kwargs' not in init_kwargs:
+            init_kwargs['tap_kwargs'] = dict()
+        tap_kwargs = init_kwargs['tap_kwargs']
+
+        if 'tap_widget' not in tap_kwargs:
+            tap_kwargs['tap_widget'] = tap_widget
+
+        if 'popup_kwargs' not in tap_kwargs:
+            tap_kwargs['popup_kwargs'] = dict()
+        popup_kwargs = tap_kwargs['popup_kwargs']
+        if 'parent' not in popup_kwargs:
+            popup_kwargs['parent'] = tap_kwargs['tap_widget']
 
-    :param action:      flow action string.
-    :param obj:         flow object (defined by app project).
-    :param key:         flow index/item_id/field_id/... (defined by app project).
-    :return:            complete flow_id string.
     """
-    assert action == '' or FLOW_ACTION_RE.fullmatch(action), \
-        f"flow action only allows lowercase letters and digits: got '{action}'"
-    assert obj == '' or FLOW_OBJECT_RE.fullmatch(obj), \
-        f"flow object only allows letters, digits and underscores: got '{obj}'"
-    cid = f'{action}{NAME_PARTS_SEP if action and obj else ""}{obj}'
-    if key:
-        cid += f'{FLOW_KEY_SEP}{key}'
-    return cid
+    init_kwargs['tap_kwargs'] = tap_kwargs = init_kwargs.get('tap_kwargs', dict())
+    tap_kwargs['tap_widget'] = tap_widget = tap_kwargs.get('tap_widget', tap_widget)
+    tap_kwargs['popup_kwargs'] = popup_kwargs = tap_kwargs.get('popup_kwargs', dict())
+    popup_kwargs['parent'] = popup_kwargs.get('parent', tap_widget)
 
 
 def flow_action(flow_id: str) -> str:
@@ -487,6 +497,24 @@ def flow_popup_class_name(flow_id: str) -> str:
     return f'{snake_to_camel(obj)}{action.capitalize()}Popup'
 
 
+def id_of_flow(action: str, obj: str = "", key: str = "") -> str:
+    """ create flow id string.
+
+    :param action:      flow action string.
+    :param obj:         flow object (defined by app project).
+    :param key:         flow index/item_id/field_id/... (defined by app project).
+    :return:            complete flow_id string.
+    """
+    assert action == '' or FLOW_ACTION_RE.fullmatch(action), \
+        f"flow action only allows lowercase letters and digits: got '{action}'"
+    assert obj == '' or FLOW_OBJECT_RE.fullmatch(obj), \
+        f"flow object only allows letters, digits and underscores: got '{obj}'"
+    cid = f'{action}{NAME_PARTS_SEP if action and obj else ""}{obj}'
+    if key:
+        cid += f'{FLOW_KEY_SEP}{key}'
+    return cid
+
+
 def register_package_images():              # pragma: no cover
     """ call from module scope of the package for to register/add image/img resources path. """
     global FOUND_IMAGES
@@ -515,6 +543,24 @@ def replace_flow_action(flow_id: str, new_action: str):
     return id_of_flow(new_action, *flow_key_split(flow_action_split(flow_id)[1]))
 
 
+def update_tap_kwargs(widget: Any, popup_kwargs: Optional[Dict[str, Any]] = None, **tap_kwargs) -> Dict[str, Any]:
+    """ update widget's tap_kwargs property and return the updated dictionary (for kv rule of tap_kwargs).
+
+    :param widget:              widget with tap_kwargs property to be updated.
+    :param popup_kwargs:        dict with items for to update popup_kwargs key of tap_kwargs
+    :param tap_kwargs:          additional tap_kwargs items to update.
+    :return:
+    """
+    handle = dict(tap_kwargs=widget.tap_kwargs)
+    ensure_tap_kwargs_refs(handle, widget)
+    new_kwargs = handle['tap_kwargs']
+    if popup_kwargs:
+        new_kwargs['popup_kwargs'].update(popup_kwargs)
+    if tap_kwargs:
+        new_kwargs.update(tap_kwargs)
+    return new_kwargs
+
+
 class MainAppBase(ConsoleApp, ABC):
     """ abstract base class for to implement a GUIApp-conform app class """
     # app states
